package edu.upc.essi.petrinets.uuid;

import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;

import org.processmininguc.utils.DeepClone;

import stateMachine.Transition.EventDepCondType;

public class Petrinet {
	
	private String m_label;
	
	private HashMap<PlaceId, Collection<TransitionId>> m_placeToTrans;
	private HashMap<TransitionId, Collection<PlaceId>> m_transToPlace;
	
	private HashMap<ArcId,PlaceToTrans> m_arcPlaceToTrans;
	private HashMap<ArcId,TransToPlace> m_arcTransToPlace;
	
	private HashMap<PlaceId, PlaceAttributes> m_placeAttributes;
	private HashMap<TransitionId, TransitionAttributes> m_transitionAttributes; 
	
	public Petrinet(String label){
		m_label = label;
		m_placeToTrans = new HashMap<>();
		m_transToPlace = new HashMap<>();
		m_placeAttributes = new HashMap<>();
		m_transitionAttributes = new HashMap<>();
		m_arcPlaceToTrans = new HashMap<>();
		m_arcTransToPlace = new HashMap<>();
	}
	
	public PlaceId addPlace(String label, int tokenNr){
		PlaceId place = new PlaceId();
		PlaceAttributes placeAtt = new PlaceAttributes();
		placeAtt.label = label;
		placeAtt.tokenNr = tokenNr;
		m_placeAttributes.put(place,placeAtt);
		return place;
	}
	
	public PlaceId addPlace(String label){
		PlaceId place = addPlace(label,0);
		return place;
	}
	
	/**
	 * Returns true if the place has been added. False otherwise (it already existed) or it wasn't contained in the Petrinet given as input.
	 * @param place
	 * @return
	 */
	public boolean addPlaceFromPetrinet(PlaceId place, Petrinet pn){
		if(m_placeAttributes.containsKey(place) || !pn.m_placeAttributes.containsKey(place)){
			return false;
		}
		else{
			PlaceAttributes placeAtt = pn.m_placeAttributes.get(place);
			m_placeAttributes.put(place,placeAtt);
			return true;
		}
	}
	
	public TransitionId addTransition(String label, boolean invisible){
		TransitionId trans = new TransitionId();
		TransitionAttributes transAtt = new TransitionAttributes();
		transAtt.label = label;
		transAtt.invisible = invisible;
		m_transitionAttributes.put(trans,transAtt);
		return trans;
	}
	
	
	public TransitionId addTransition(String label){
		return addTransition(label,false);
	}
	
	
	/**
	 * Returns true if the place has been added. False otherwise (it already existed) or it wasn't contained in the Petrinet given as input.
	 * @param place
	 * @return
	 */
	public boolean addTransitionFromPetrinet(TransitionId trans, Petrinet pn){
		if(m_transitionAttributes.containsKey(trans) || !pn.m_transitionAttributes.containsKey(trans)){
			return false;
		}
		else{
			TransitionAttributes transAtt = pn.m_transitionAttributes.get(trans);
			m_transitionAttributes.put(trans,transAtt);
			return true;
		}
	}
	
	public ArcId addArc(PlaceId place, TransitionId trans, EventDepCondType ster){
		ArcId newArc = addArc(place,trans);
		PlaceToTrans pTT = m_arcPlaceToTrans.get(newArc);
		pTT.evDepCond = ster;
		return newArc;
	}
	
	
	public ArcId addArc(PlaceId place, TransitionId trans){
		if(!m_placeAttributes.containsKey(place) || !m_transitionAttributes.containsKey(trans)){
			//TODO: Throws exception
		}
		//Duplicats tractats com a les llistes
		ArcId arc = new ArcId();
		if(m_placeToTrans.containsKey(place)){
			Collection<TransitionId> transCol = m_placeToTrans.get(place);
			transCol.add(trans);
		}
		else{
			LinkedList<TransitionId> transList = new LinkedList<>();
			transList.add(trans);
			m_placeToTrans.put(place,transList);
		}
		PlaceToTrans pTT = new PlaceToTrans();
		pTT.source = place;
		pTT.target = trans;
		pTT.type = ArcType.Standard;
		pTT.evDepCond = null;
		m_arcPlaceToTrans.put(arc, pTT);
		return arc;
	}
	
	public ArcId addArc(TransitionId trans, PlaceId place, EventDepCondType ster){
		ArcId newArc = addArc(trans,place);
		TransToPlace ttp = m_arcTransToPlace.get(newArc);
		ttp.evDepCond = ster;
		return newArc;
	}
	
	public ArcId addArc(TransitionId trans, PlaceId place){
		if(!m_placeAttributes.containsKey(place) || !m_transitionAttributes.containsKey(trans)){
			//TODO: Throws exception
		}
		ArcId arc = new ArcId();
		if(m_transToPlace.containsKey(trans)){
			Collection<PlaceId> placeCol = m_transToPlace.get(trans);
			if(placeCol.contains(place)){
				System.out.println("Error!!!");
			}
			placeCol.add(place);
		}
		else{
			LinkedList<PlaceId> placeList = new LinkedList<>();
			placeList.add(place);
			m_transToPlace.put(trans,placeList);
		}
		TransToPlace tTP = new TransToPlace();
		tTP.source = trans;
		tTP.target = place;
		tTP.type = ArcType.Standard;
		tTP.evDepCond = null;
		m_arcTransToPlace.put(arc, tTP);
		return arc;
	}
	
	public ArcId addInhibitorArc(PlaceId source, TransitionId target){
		ArcId resultingArc = addArc(source,target);
		PlaceToTrans pTT = m_arcPlaceToTrans.get(resultingArc);
		pTT.type = ArcType.Inhibitor;
		return resultingArc;
	}
	
	public void removeTransition(TransitionId trans){
		m_transitionAttributes.remove(trans);
		m_transToPlace.remove(trans);
		for(Iterator<PlaceId> it = m_placeToTrans.keySet().iterator(); it.hasNext();){ //To avoid concurrency issues for mod.
			PlaceId place = it.next();
			Collection<TransitionId> transCol = m_placeToTrans.get(place);
			if (transCol.contains(trans)){
				transCol.remove(trans);
				if(transCol.isEmpty()){
					//m_placeToTrans.remove(place);
					it.remove();
				}
			}
		}
		for(Iterator<ArcId> it = m_arcPlaceToTrans.keySet().iterator(); it.hasNext();){
			ArcId arc = it.next();
			PlaceToTrans pTT = m_arcPlaceToTrans.get(arc);
			if (pTT.target.equals(trans)){
				it.remove();
				//m_arcPlaceToTrans.remove(arc);
			}
		}
		for(Iterator<ArcId> it = m_arcTransToPlace.keySet().iterator(); it.hasNext();){
			ArcId arc = it.next();
			TransToPlace tTP = m_arcTransToPlace.get(arc);
			if (tTP.source.equals(trans)){
				//m_arcPlaceToTrans.remove(arc);
				it.remove();
			}
		}
	}
	
	public void removeArc(ArcId arc){
		if(m_arcPlaceToTrans.containsKey(arc)){
			PlaceToTrans pTT = m_arcPlaceToTrans.get(arc);
			PlaceId place = pTT.source;
			Collection<TransitionId> transCol = m_placeToTrans.get(place);
			transCol.remove(pTT.target);
			if(transCol.isEmpty()){
				m_placeToTrans.remove(place);
			}
			m_arcPlaceToTrans.remove(arc);
		}
		else if(m_arcTransToPlace.containsKey(arc)){
			TransToPlace tTP = m_arcTransToPlace.get(arc);
			TransitionId trans = tTP.source;
			Collection<PlaceId> placeCol = m_transToPlace.get(trans);
			placeCol.remove(tTP.target);
			if(placeCol.isEmpty()){
				m_transToPlace.remove(trans);
			}
			m_arcTransToPlace.remove(arc);
		}
	}
	
	public void removePlace(PlaceId place){
		m_placeAttributes.remove(place);
		m_placeToTrans.remove(place);
		for(Iterator<TransitionId> it = m_transToPlace.keySet().iterator(); it.hasNext();){
			TransitionId trans = it.next();
			Collection<PlaceId> placeCol = m_transToPlace.get(trans);
			if (placeCol.contains(place)){ //Canviat aqui!!
				placeCol.remove(place);
				if(placeCol.isEmpty()){
					//m_placeToTrans.remove(place);
					it.remove();
				}
			}
		}
		for(Iterator<ArcId> it = m_arcPlaceToTrans.keySet().iterator(); it.hasNext();){
			ArcId arc = it.next();
			PlaceToTrans pTT = m_arcPlaceToTrans.get(arc);
			if (pTT.source.equals(place)){
				it.remove();
			}
		}
		for(Iterator<ArcId> it = m_arcTransToPlace.keySet().iterator(); it.hasNext();){
			ArcId arc = it.next();
			TransToPlace tTP = m_arcTransToPlace.get(arc);
			if (tTP.target.equals(place)){
				it.remove();
			}
		}
	}
	
	public String getPetrinetLabel(){
		return m_label;
	}
	
	public Collection<PlaceId> getPlaces(){
		return m_placeAttributes.keySet();
	}
	
	public Collection<TransitionId> getTransitions(){
		return m_transitionAttributes.keySet();
	}
	
	public boolean isTransitionInvisible(TransitionId transId){
		return m_transitionAttributes.get(transId).invisible;
	}
	
	public Collection<ArcId> getArcs(){
		Collection<ArcId> arcList = new LinkedList<>();
		arcList.addAll(m_arcPlaceToTrans.keySet());
		arcList.addAll(m_arcTransToPlace.keySet());
		return arcList;
	}
	
	/**
	 * @param source
	 * @param target
	 * @return ArcId corresponding to the inputs given. Null if there is no Arc.
	 */
	public ArcId getArc(PlaceId source, TransitionId target){
		System.out.println("********");
		System.out.println("getArc()");
		System.out.println("source" + getPlaceLabel(source) + " target: " + getTransitionLabel(target));
		ArcId result = null;
		Collection<ArcId> arcList = m_arcPlaceToTrans.keySet();
		for(ArcId currentArc: arcList){
			PlaceToTrans placeTrans = m_arcPlaceToTrans.get(currentArc);
			if (placeTrans.source.equals(source) && placeTrans.target.equals(target)){
				result = currentArc;
			}
		}
		System.out.println("********");
		return result;
	}
	
	/**
	 * 
	 * @param source
	 * @param target
	 * @return ArcId corresponding to the inputs given. Null if there is no Arc
	 */
	public ArcId getArc(TransitionId source, PlaceId target){
		System.out.println("********");
		System.out.println("getArc()");
		System.out.println("source" + getTransitionLabel(source) + " target: " + getPlaceLabel(target));
		ArcId result = null;
		Collection<ArcId> arcList = m_arcTransToPlace.keySet();
		for(ArcId currentArc: arcList){
			TransToPlace transPlace = m_arcTransToPlace.get(currentArc);
			if (transPlace.source.equals(source) && transPlace.target.equals(target)){
				result = currentArc;
			}
		}
		return result;
	}
	
	/**
	 * Returns the type of the given Arc. If there is no such Arc in the Petrinet, it returns null.
	 * @param arc
	 * @return
	 */
	public ArcType getArcType(ArcId arc){
		if(m_arcTransToPlace.containsKey(arc)){
			return m_arcTransToPlace.get(arc).type;
		}
		else if (m_arcPlaceToTrans.containsKey(arc)){
			return m_arcPlaceToTrans.get(arc).type;
		}
		return null;
	}
	
	public String getPlaceLabel(PlaceId place){
		if(!m_placeAttributes.containsKey(place)){
			//TODO: Throw exception!!
			System.err.println("m_placeAttributes does not contain the given place");
		}
		return m_placeAttributes.get(place).label;
	}
	
	public int getPlaceTokens(PlaceId place){
		return m_placeAttributes.get(place).tokenNr;
	}
	
	public void setPlaceTokens(PlaceId place, int tokens){
		m_placeAttributes.get(place).tokenNr = tokens;
	}
	
	public String getTransitionLabel(TransitionId trans){
		return m_transitionAttributes.get(trans).label;
	}
	
	/**
	 * Returns a collection of places which could be considered "Initial". That is, they don't have any input edges.
	 * @return
	 */
	public Collection<PlaceId> getInitialPlaces(){
		Collection<PlaceId> placeCol = new LinkedList<>(m_placeAttributes.keySet());
		Collection<PlaceId> placesWithIncomingArcs = new LinkedList<>();
		for(TransitionId trans: m_transToPlace.keySet()){
			placesWithIncomingArcs.addAll(m_transToPlace.get(trans));
		}
		placeCol.removeAll(placesWithIncomingArcs);
		return placeCol;
	}
	
	public Collection<PlaceId> getFinalPlaces(){
		Collection<PlaceId> placeCol = new LinkedList<>(m_placeAttributes.keySet());
		Collection<PlaceId> placesWithOutgoingArcs = m_placeToTrans.keySet();
		placeCol.removeAll(placesWithOutgoingArcs);
		return placeCol;
	}
	
	public Collection<TransitionId> getNextTransitions(PlaceId place){
		Collection<TransitionId> transIdCol;
		if(m_placeToTrans.containsKey(place)){
			transIdCol = m_placeToTrans.get(place);
		}
		else{
			transIdCol = new LinkedList<>();
		}
		return transIdCol;
	}
	
	public Collection<TransitionId> getPreviousTransitions(PlaceId place){
		Collection<TransitionId> transIdCol = new LinkedList<>();
		for(TransitionId trans: m_transToPlace.keySet()){
			if(m_transToPlace.get(trans).contains(place)){
				transIdCol.add(trans);
			}
		}
		return transIdCol;
	}
	
	public Collection<PlaceId> getNextPlaces(TransitionId trans){
		Collection<PlaceId> placeIdCol = m_transToPlace.get(trans);
		if(m_transToPlace.get(trans) == null){
			placeIdCol = new LinkedList<PlaceId>();
		}
		return placeIdCol;
	}
	
	public Collection<PlaceId> getPreviousPlaces(TransitionId trans){
		Collection<PlaceId> placeIdCol = new LinkedList<>();
		for(PlaceId place: m_placeToTrans.keySet()){
			if(m_placeToTrans.get(place).contains(trans)){
				placeIdCol.add(place);
			}
		}
		return placeIdCol;
	}
	
	/**
	 * Adds a stereotype to the given arc. If the arc already had a stereotype (i.e. it was different than null), it does not make any changes
	 * 
	 * @return - boolean - true if the stereotype has been added, false otherwise
	 */
	public boolean addEventDepCondToArc(ArcId arc, EventDepCondType evDepCond){
		boolean result = false;
		if(m_arcPlaceToTrans.containsKey(arc)){
			PlaceToTrans pTT = m_arcPlaceToTrans.get(arc);
			if(pTT.evDepCond == null){
				pTT.evDepCond = evDepCond;
				result = true;
			}
		}
		else if(m_arcTransToPlace.containsKey(arc)){
			TransToPlace tTP = m_arcTransToPlace.get(arc);
			if(tTP.evDepCond == null){
				tTP.evDepCond = evDepCond;
				result = true;
			}
		}
		return result;
	}
	
	/**
	 * Returns the event-dependent condition associated to certain arc. Will return null if the petri net does not contain the arc, or if there is no event-dependent
	 * condition.
	 * @param arc
	 * @return
	 */
	public EventDepCondType getEventDepCondForArc(ArcId arc){
		if (m_arcPlaceToTrans.containsKey(arc)){
			return m_arcPlaceToTrans.get(arc).evDepCond;
		}
		else if(m_arcTransToPlace.containsKey(arc)){
			return m_arcTransToPlace.get(arc).evDepCond;
		}
		return null;
	}
	
	
	/**
	 * Makes a copy of the Petrinet on which the function is called
	 * @return A copy of the Petrinet
	 */
	public Petrinet clonePetrinet(){
		Petrinet pnCopy = new Petrinet(this.m_label);
		HashMap<ArcId,PlaceToTrans> arcPlaceToTrans = DeepClone.deepClone(this.m_arcPlaceToTrans);
		HashMap<ArcId,TransToPlace> arcTransToPlace = DeepClone.deepClone(this.m_arcTransToPlace);
		HashMap<PlaceId,PlaceAttributes> placeAttributes = DeepClone.deepClone(this.m_placeAttributes);
		HashMap<TransitionId,TransitionAttributes> transAttributes = DeepClone.deepClone(this.m_transitionAttributes);
		HashMap<PlaceId,Collection<TransitionId>> placeToTrans = DeepClone.deepClone(this.m_placeToTrans);
		HashMap<TransitionId,Collection<PlaceId>> transToPlace = DeepClone.deepClone(this.m_transToPlace);
		
		pnCopy.m_arcPlaceToTrans = arcPlaceToTrans;
		pnCopy.m_arcTransToPlace = arcTransToPlace;
		pnCopy.m_placeAttributes = placeAttributes;
		pnCopy.m_transitionAttributes = transAttributes;
		pnCopy.m_placeToTrans = placeToTrans;
		pnCopy.m_transToPlace = transToPlace;
		
		return pnCopy;
	}
	
	/**
	 * Makes a copy of the Petrinet on which the function is called. To do so, 
	 * @return
	 */
	public Petrinet clonePetrinetWithNewIds(){
		Petrinet pnCopy = new Petrinet(this.m_label);
		HashMap<ArcId,PlaceToTrans> arcPlaceToTrans = new HashMap<>();
		HashMap<ArcId,TransToPlace> arcTransToPlace = new HashMap<>();
		HashMap<PlaceId,PlaceAttributes> placeAttributes = new HashMap<>();
		HashMap<TransitionId,TransitionAttributes> transAttributes = new HashMap<>();
		HashMap<PlaceId,Collection<TransitionId>> placeToTrans = new HashMap<>();
		HashMap<TransitionId,Collection<PlaceId>> transToPlace = new HashMap<>();
		
		//Temporarily stores the correspondence between the Places, Transitions and Arcs in the original hashmap and the new one.
		HashMap<PlaceId,PlaceId> tempPlaceCorrespond = new HashMap<>();
		HashMap<TransitionId,TransitionId> tempTransCorrespond = new HashMap<>();
		//HashMap<ArcId,ArcId> tempArcCorrespond = new HashMap<>();
		
		//We create the new places that will be equivalent to the previous ones	
		for(PlaceId place: this.m_placeAttributes.keySet()){
			PlaceId newPlace = new PlaceId();
			tempPlaceCorrespond.put(place, newPlace);
		}
		
		//We do the same thing for the transitions
		for(TransitionId trans: this.m_transitionAttributes.keySet()){
			TransitionId newTransition = new TransitionId();
			tempTransCorrespond.put(trans, newTransition);
		}
		
		//We copy the attributes for the places and add them to the corresponding list
		for(PlaceId place: this.m_placeAttributes.keySet()){
			PlaceAttributes placeAtt = this.m_placeAttributes.get(place);
			PlaceAttributes placeAttCopy = new PlaceAttributes();
			placeAttCopy.label = placeAtt.label;
			placeAttCopy.tokenNr = placeAtt.tokenNr;
			PlaceId correspPlace = tempPlaceCorrespond.get(place);
			placeAttributes.put(correspPlace,placeAttCopy);
		}
		
		//We copy the attributes for the transitions and add them to the corresponding list
		for(TransitionId trans: this.m_transitionAttributes.keySet()){
			TransitionAttributes transAtt = this.m_transitionAttributes.get(trans);
			TransitionAttributes transAttCopy = new TransitionAttributes();
			transAttCopy.label = transAtt.label;
			transAttCopy.invisible = transAtt.invisible;
			TransitionId correspTrans = tempTransCorrespond.get(trans);
			transAttributes.put(correspTrans, transAttCopy);
		}
		
		//We copy the arcs that go from place to transition
		for(PlaceId place: this.m_placeToTrans.keySet()){
			PlaceId correspPlace = tempPlaceCorrespond.get(place);
			placeToTrans.put(correspPlace, new LinkedList<TransitionId>());
			for(TransitionId trans: this.m_placeToTrans.get(place)){
				TransitionId correspTrans = tempTransCorrespond.get(trans);
				Collection<TransitionId> correspTransList = placeToTrans.get(correspPlace);
				correspTransList.add(correspTrans);
			}
		}
		
		//We copy the arcs that go from transition to place
		for(TransitionId trans: this.m_transToPlace.keySet()){
			TransitionId correspTrans = tempTransCorrespond.get(trans);
			transToPlace.put(correspTrans, new LinkedList<PlaceId>());
			for(PlaceId place: this.m_transToPlace.get(trans)){
				PlaceId correspPlace = tempPlaceCorrespond.get(place);
				Collection<PlaceId> correspPlaceList = transToPlace.get(correspTrans);
				correspPlaceList.add(correspPlace);
			}
		}
		
		for(ArcId arc: this.m_arcPlaceToTrans.keySet()){
			ArcId correspArc = new ArcId();
			PlaceToTrans pTT = m_arcPlaceToTrans.get(arc);
			PlaceToTrans correspPTT = new PlaceToTrans();
			correspPTT.source = tempPlaceCorrespond.get(pTT.source);
			correspPTT.target = tempTransCorrespond.get(pTT.target);
			correspPTT.type = pTT.type;
			correspPTT.evDepCond = pTT.evDepCond;
			arcPlaceToTrans.put(correspArc,correspPTT);
		}
		
		for(ArcId arc: this.m_arcTransToPlace.keySet()){
			ArcId correspArc = new ArcId();
			TransToPlace tTP = m_arcTransToPlace.get(arc);
			TransToPlace correspTTP = new TransToPlace();
			correspTTP.source = tempTransCorrespond.get(tTP.source);
			correspTTP.target = tempPlaceCorrespond.get(tTP.target);
			correspTTP.type = tTP.type;
			correspTTP.evDepCond = tTP.evDepCond;
			arcTransToPlace.put(correspArc, correspTTP);
		}
		
		pnCopy.m_arcPlaceToTrans = arcPlaceToTrans;
		pnCopy.m_arcTransToPlace = arcTransToPlace;
		pnCopy.m_label = this.m_label;
		pnCopy.m_placeAttributes = placeAttributes;
		pnCopy.m_placeToTrans = placeToTrans;
		pnCopy.m_transitionAttributes = transAttributes;
		pnCopy.m_transToPlace = transToPlace;
		
		return pnCopy;
		
	}
	
	private class PlaceAttributes{
		public String label;
		public int tokenNr;
	}

	private class TransitionAttributes{
		public String label;
		public boolean invisible;
	}
	
	private class TransToPlace{
		public TransitionId source;
		public PlaceId target;
		public ArcType type;
		public EventDepCondType evDepCond;
	}
	
	private class PlaceToTrans{
		public PlaceId source;
		public TransitionId target;
		public ArcType type;
		public EventDepCondType evDepCond;
	}
	
}



