// Copyright (C) 2014 by Jorge Munoz-Gama. All rights reserved. 
// Released under the terms of the Lesser GNU General Public License (L-GPL) version 2 or later.

package org.processmininguc.balsa.algorithms;

import java.util.LinkedHashSet;
import java.util.Set;

import org.processmining.framework.util.Pair;
import org.processmining.models.graphbased.directed.petrinet.InhibitorNet;
import org.processmining.models.graphbased.directed.petrinet.elements.Arc;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.models.graphbased.directed.petrinet.impl.PetrinetFactory;
import org.processmining.models.semantics.petrinet.Marking;
import org.processmininguc.balsa.models.Artifact;
import org.processmininguc.balsa.models.Balsa;
import org.processmininguc.balsa.models.StateMachineDiagram;
import org.processmininguc.balsa.models.StateMachineDiagramCondition;
import org.processmininguc.balsa.models.StateMachineDiagramEvent;
import org.processmininguc.balsa.models.StateMachineDiagramState;
import org.processmininguc.balsa.models.StateMachineDiagramTransition;
import org.processmininguc.balsa.parameters.ConvertBalsaArtifactToPetriNetConfiguration;

public class OLDBalsaArtifactToPetriNetConverter {
	
	//Initial Parameters
	private Balsa balsa;
	private ConvertBalsaArtifactToPetriNetConfiguration configuration;
	private Artifact artifactToConvert;
	
	//Intermediate: Step 1
	private Set<StateMachineDiagramEvent> transitionsStep1;
	private Set<StateMachineDiagramState> placesStep1;
	private Set<StateMachineDiagramState> tokensStep1;
	private Set<Pair<StateMachineDiagramState, StateMachineDiagramEvent>> arcsPlaceTransitionStep1;
	private Set<Pair<Pair<StateMachineDiagramEvent, StateMachineDiagramState>,StateMachineDiagramCondition>> arcsTransitionPlaceStep1;	
	
	//Intermediate: Step 2
	
	
	//Final Results
	private InhibitorNet convertedNet;
	private Marking convertedInitialMarking;
	
	
	public OLDBalsaArtifactToPetriNetConverter(Balsa balsa, ConvertBalsaArtifactToPetriNetConfiguration configuration) {
		this.balsa = balsa;
		this.configuration = configuration;
		this.artifactToConvert = configuration.getArtifactToConvert();
		convert();
	}
	
	private void convert(){
		step1();
		if(artifactToConvert.isContextIndependent()) step2ContextIndependent();
		else step2ContextDependent();
		createNet();
	}
	

	private void step1(){
		
		StateMachineDiagram ts = artifactToConvert.getStateMachineDiagram();
		
		//Places and initial marking
		placesStep1 = new LinkedHashSet<StateMachineDiagramState>(ts.getStateList());	
		tokensStep1 = new LinkedHashSet<StateMachineDiagramState>();
		tokensStep1.add(ts.getInitialState());
		
		//Transitions
		transitionsStep1 = new LinkedHashSet<StateMachineDiagramEvent>(ts.getEventList());
		arcsPlaceTransitionStep1 = new LinkedHashSet<Pair<StateMachineDiagramState, StateMachineDiagramEvent>>();
		arcsTransitionPlaceStep1 = new LinkedHashSet<Pair<Pair<StateMachineDiagramEvent, StateMachineDiagramState>,StateMachineDiagramCondition>>();
		
		for (StateMachineDiagramTransition transition: ts.getTransitionList()){
			if (!transition.isInitial() && !transition.isFinal()){ //Not consider initial/final arcs
				StateMachineDiagramState s = transition.getSourceNode();
				StateMachineDiagramState t = transition.getTargetNode();
				StateMachineDiagramEvent e = transition.getEvent();
				StateMachineDiagramCondition c = transition.getGuardCondition();
				
				Pair<StateMachineDiagramState, StateMachineDiagramEvent> arcPT = 
						new Pair<StateMachineDiagramState, StateMachineDiagramEvent>(s,e);
				arcsPlaceTransitionStep1.add(arcPT);
				
				Pair<Pair<StateMachineDiagramEvent, StateMachineDiagramState>,StateMachineDiagramCondition> arcTP =
						new Pair<Pair<StateMachineDiagramEvent, StateMachineDiagramState>,StateMachineDiagramCondition>(
							new Pair<StateMachineDiagramEvent, StateMachineDiagramState>(e, t), c);
				arcsTransitionPlaceStep1.add(arcTP);
				
			}
		}
		
		
	}
	
	private void step2ContextIndependent(){
		
	}
	
	private void step2ContextDependent(){
		
	}
	
	private void createNet() {
		// TODO example net
		convertedNet = PetrinetFactory.newInhibitorNet(artifactToConvert.getName());
		Place p00 = convertedNet.addPlace("p00");
		Place p01 = convertedNet.addPlace("p01");
		Transition t00 = convertedNet.addTransition("t00");
		Arc a00 = convertedNet.addArc(p00, t00);
		Arc a01 = convertedNet.addArc(t00, p01);
		
		convertedInitialMarking = new Marking();
		convertedInitialMarking.add(p00);
	}

	public InhibitorNet getConvertedNet() {
		return this.convertedNet;
	}

	public Marking getConvertedInitialMarking() {
		return this.convertedInitialMarking;
	}
		

}
