// Copyright (C) 2014 by Jorge Munoz-Gama. All rights reserved. 
// Released under the terms of the Lesser GNU General Public License (L-GPL) version 2 or later.

package org.processmininguc.balsa.models;

import java.util.List;

/**
 * Represents any node in the state machine diagram.
 * @author Montse
 *
 */
public interface StateMachineDiagramState {
	
	/**
	 * Returns the list of incoming transitions in the current state (i.e. node).
	 * @return
	 */
    public List<StateMachineDiagramTransition> getIncomingTransition();
    
    /**
     * Returns the list of outgoing transitions in the current state (i.e. node).
     * @return
     */
    public List<StateMachineDiagramTransition> getOutgoingTransition();
    
    /**
     * Returns the next states (i.e. nodes) of the current state (i.e. node).
     * @return
     */
    public List<StateMachineDiagramState> getNextStates();
    
    /**
     * Returns the previous states (i.e. nodes) of the current state (i.e. node).
     * @return
     */
    public List<StateMachineDiagramState> getPreviousStates();
    
    /**
     * Returns a boolean informing of whether the state (i.e. node) is a state or not. 
     * @return boolean.
     */
    public boolean isState();
    
    /**
     * Returns a boolean informing of whether the state (i.e. node) is an initial pseudostate or not. 
     */
    public boolean isInitialPseudostate();
    
    /**
     * Returns a boolean informing of whether the state (i.e. node) is a final state or not. 
     * @return boolean.
     */
    public boolean isFinalState();
    
    /**
     * Returns a boolean informing of whether the state (i.e. node) is a pseudostate or not. 
     * @return boolean.
     */
    public boolean isPseudostate();

}
