package org.processmininguc.balsa.models.impl.smadu;

import java.util.List;

import parser.StateMachineFactoryInterface;
import stateMachine.Effect;
import stateMachine.Event;
import stateMachine.FinalState;
import stateMachine.Pseudostate;
import stateMachine.PseudostateKind;
import stateMachine.State;
import stateMachine.StateMachine;
import stateMachine.Transition;
import stateMachine.Transition.EventDepCondType;
import stateMachine.Trigger;
import stateMachine.Vertex;

public class StateMachineDiagramFactory implements StateMachineFactoryInterface {

	public Effect createEffect(String name) {
 		return new StateMachineDiagramEffectImpl(name);
	}

	public Event createEvent(String name) {
		return new StateMachineDiagramEventImpl(name);
	}

	public FinalState createFinalState() {
		return new StateMachineDiagramFinalStateImpl();
	}

	public Pseudostate createInitialPseudostate() {
		return new StateMachineDiagramPseudostateImpl(PseudostateKind.initial);
	}

	/**
	 * !!State in the library corresponds to a RealState in this plugin.
	 */
	public State createState(String name) {
		return new StateMachineDiagramRealStateImpl(name);
	}

	public StateMachine createStateMachine(String name) {
		return new StateMachineDiagramImpl(name);
	}

	public StateMachine createStateMachine(String name, Pseudostate iniState) {
		//Nota: Pseudostate hauria de ser del tipus especfic del plugin
		return new StateMachineDiagramImpl(name, iniState);
	}

	public Transition createTransition() {
		return new StateMachineDiagramTransitionImpl();
	}

	public Transition createTransition(Vertex source, Vertex target) {
		return new StateMachineDiagramTransitionImpl(source,target);
	}

	public Transition createTransition(Vertex source, Vertex target, List<Trigger> triggerList, String guard, EventDepCondType evtDepCond,
			Effect effect) {
		return new StateMachineDiagramTransitionImpl(source,target,triggerList,guard,evtDepCond,effect);
	}

	public Trigger createTrigger(Event event) {
		return new StateMachineDiagramTriggerImpl(event);
	}

}
