package org.processmininguc.balsa.models.impl.smadu;

import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;

import org.processmininguc.balsa.models.StateMachineDiagram;
import org.processmininguc.balsa.models.StateMachineDiagramEvent;
import org.processmininguc.balsa.models.StateMachineDiagramState;
import org.processmininguc.balsa.models.StateMachineDiagramTransition;
import org.processmininguc.utils.Utils;

import stateMachine.Pseudostate;
import stateMachine.State;
import stateMachine.StateMachine;
import stateMachine.Transition;

public class StateMachineDiagramImpl extends StateMachine implements StateMachineDiagram {
	
	public StateMachineDiagramImpl(String name) {
		super(name);
	}
	
	public StateMachineDiagramImpl(String name, Pseudostate iniState){
		super(name,iniState);
	}
	
	public StateMachineDiagramImpl(String name, StateMachineDiagramPseudostateImpl iniState){
		super(name,iniState);
	}
	
//	public StateMachineDiagramImpl(StateMachine smd){
//		super(smd.getName());
//		StateMachineDiagramState smdIniState = new StateMachineDiagramPseudostateImpl(); //By default is an initial state.
//		//smd.getIniState().getOutgoing()
//	}

	/**
	 * Returns a list of the states in the state machine diagram. However, the list of states does NOT include
	 * any type of pseudostate, such as the initial node or the final nodes.
	 */
	public List<StateMachineDiagramState> getStateList() {
		HashSet<State> stateSet = super.getStates();
		LinkedList<StateMachineDiagramState> smdStateList = new LinkedList<>();
		for(State s: stateSet){
			smdStateList.add((StateMachineDiagramRealStateImpl)s);
		}
		return smdStateList;
	}
	
	/**
	 * Returns a list of the states in the state machine diagram, including the initial state but not the final one.
	 * TODO: Improve this!!! Should probably return final state as well.
	 */
	public List<StateMachineDiagramState> getAllStatesList() {
		List<StateMachineDiagramState> smdStateList = getStateList();
		smdStateList.add(getInitialState());
		return smdStateList;
	}

	public StateMachineDiagramState getInitialState() {
		StateMachineDiagramPseudostateImpl smdState = (StateMachineDiagramPseudostateImpl) getIniState();
		return smdState;
	}

	public List<StateMachineDiagramTransition> getTransitionList() {
		HashSet<Transition> transSet = super.getTransitions();
		LinkedList<StateMachineDiagramTransition> smdTransList = new LinkedList<>();
		for(Transition t: transSet){
			smdTransList.add((StateMachineDiagramTransition)t);
		}
		return smdTransList;
	}

	public List<StateMachineDiagramEvent> getEventList() {
		return Utils.convert(getEventList());
	}

}
