package org.processmininguc.balsa.models.structures;

import stateMachine.Transition.EventDepCondType;

/**
 * Structure used to represent the simple changes made by an activity diagram.
 * 
 * @author Montse
 *
 */
public class SimpleADChange {

	private String m_context; //Represents the artifact from whose perspective the change is made.
	private String m_targetArtifact; //Repesents the artifact whose quantity is changed.
	private OpType m_operation; //Represents the type of operation performed by the activity diagram, such as increment or decrement the number of artifacts.
	private int m_number; //Quantity by which the number of targetArtifacts related to the context artifact is modified.
	private EventDepCondType m_evDepCond; //Stereotype/Tag/Event-dependent condition which determines when the change is made. May be null if it applies to all cases or there are no tags/stereotypes/event-dependent conditions.
	
	public SimpleADChange(String context, String targetArtifact, OpType op, int number, EventDepCondType evDepCond){
		m_context = context;
		m_operation = op;
		m_targetArtifact = targetArtifact;
		m_number = number;
		m_evDepCond = evDepCond;
	}
	
	public String getContext(){
		return m_context;
	}
	
	public String getTargetArtifact(){
		return m_targetArtifact;
	}
	
	public OpType getOperationType(){
		return m_operation;
	}
	
	public int getNumber(){
		return m_number;
	}
	
	public EventDepCondType getEventDepCondType(){
		return m_evDepCond;
	}
}
