// Copyright (C) 2014 by Jorge Munoz-Gama. All rights reserved. 
// Released under the terms of the Lesser GNU General Public License (L-GPL) version 2 or later.

package org.processmininguc.balsa.plugins;

import org.deckfour.uitopia.api.event.TaskListener.InteractionResult;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginCategory;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.models.connections.petrinets.behavioral.InitialMarkingConnection;
import org.processmining.models.graphbased.directed.petrinet.InhibitorNet;
import org.processmining.models.semantics.petrinet.Marking;
import org.processmininguc.balsa.algorithms.BalsaArtifactToPetriNetConverter;
import org.processmininguc.balsa.dialogs.ConvertBalsaArtifactToPetriNetDialog;
import org.processmininguc.balsa.help.ConvertBalsaArtifactToPetriNetHelp;
import org.processmininguc.balsa.models.Balsa;
import org.processmininguc.balsa.parameters.ConvertBalsaArtifactToPetriNetConfiguration;

@Plugin(
		name = "Convert Balsa Artifact To Petri Net", 
		parameterLabels = { "Balsa Artifacts", "Configuration" },  
		returnLabels = { "Petri Net with Inhibitor Arcs", "Initial Marking" },
		returnTypes = { InhibitorNet.class, Marking.class },
		categories = {PluginCategory.Analytics, PluginCategory.ConformanceChecking},
		keywords = {"BALSA", "UML", "Artifact", "Petri Net", "Conformance Checking"},
		help = ConvertBalsaArtifactToPetriNetHelp.PLUGIN_DESCRIPTION,
		userAccessible = true
		)

public class ConvertBalsaArtifactToPetriNetPlugin {

	private Object[] convertPrivatePlugin(PluginContext context, 
			Balsa balsa, ConvertBalsaArtifactToPetriNetConfiguration configuration) {
		BalsaArtifactToPetriNetConverter converter = new BalsaArtifactToPetriNetConverter(balsa, configuration);
		converter.getConvertedInitialMarking();
		context.addConnection(new InitialMarkingConnection(converter.getConvertedNet(), converter.getConvertedInitialMarking()));
		return new Object[] {converter.getConvertedNet(), converter.getConvertedInitialMarking() };
	}


	
	@PluginVariant(variantLabel = "Convert Balsa Artifact To Petri Net - configuration", requiredParameterLabels = {0, 1})
	public Object[] convertConfiguredPlugin(PluginContext context, Balsa balsa, 
			ConvertBalsaArtifactToPetriNetConfiguration configuration) {
	    return convertPrivatePlugin(context, balsa, configuration);
	}
	

	@UITopiaVariant(
			affiliation = "UPC Barcelona & UC Chile", 
			author = "M.Estanyol, J.Munoz-Gama, et al.", 
			email = "estanyol"+ (char) 0x40 + "essi.upc.edu",
			uiLabel = UITopiaVariant.USEPLUGIN,
			pack = "Specifact"
			)
	@PluginVariant(variantLabel = "Convert Balsa Artifact To Petri Net - default", requiredParameterLabels = {0})
	public Object[] convertDefaultPlugin(PluginContext context, Balsa balsa) {
		// Get the default configuration.
		ConvertBalsaArtifactToPetriNetConfiguration configuration = new ConvertBalsaArtifactToPetriNetConfiguration(balsa);
	    return convertPrivatePlugin(context, balsa, configuration);
	}
	

	@UITopiaVariant(
			affiliation = "UPC Barcelona & UC Chile", 
			author = "M.Estanyol, J.Munoz-Gama, et al.", 
			email = "estanyol"+ (char) 0x40 + "essi.upc.edu",
			uiLabel = UITopiaVariant.USEPLUGIN,
			pack = "Specifact"
			)
	@PluginVariant(variantLabel = "Convert Balsa Artifact To Petri Net, dialog", requiredParameterLabels = {0})
	public Object[] convertDialogPlugin(UIPluginContext context, Balsa balsa) {
		// Get the default configuration.
		ConvertBalsaArtifactToPetriNetConfiguration configuration = new ConvertBalsaArtifactToPetriNetConfiguration(balsa);
	    // Get a dialog for this configuration.
		ConvertBalsaArtifactToPetriNetDialog dialog = new ConvertBalsaArtifactToPetriNetDialog(context, balsa, configuration);
	    // Show the dialog. User can now change the configuration.
	    InteractionResult result = context.showWizard("Settings", true, true, dialog);
	    // User has close the dialog.
	    if (result == InteractionResult.FINISHED) {
			// Do the heavy lifting.
	    	return convertPrivatePlugin(context, balsa, configuration);
	    }
	    // Dialog got canceled.
	    return null;
	}	
}
