package org.processmininguc.parser;

import org.processmininguc.balsa.models.structures.ComparisonType;

public class GuardProcessor {
	
	private static final String nrId = "#";
	
	public static ProcessedGuard processGuard(String guard){
		ProcessedGuard pg;
		if(guard == null || guard == ""){
			pg = new ProcessedGuard(false, "Empty guard");
		}
		else if (guard.contains(nrId)){
			pg = processArithmetic(guard);
		}
		else {
			pg = new ProcessedGuard(false, "Not valid processable guard"); 
		}
		return pg;
	}
	
	/**
	 * 
	 * @param guard
	 */
	private static ProcessedGuard processArithmetic(String guard){
		ProcessedGuard pg;
		String guardNoSpaces = guard.replaceAll("\\s","");
		String splitSymbol;
		ComparisonType compType;
		if(guard.contains("=")){
			if(guard.contains(">")){
				splitSymbol = ">=";
				compType = ComparisonType.GREATER_EQUALS;
			}
			else if(guard.contains("<")){
				splitSymbol = "<=";
				compType = ComparisonType.LESS_EQUALS;
			}
			else{
				//It only contains equality comparison
				splitSymbol = "=";
				compType = ComparisonType.EQUALS;
			}
		}
		else if(guard.contains("<")){
			splitSymbol = "<";
			compType = ComparisonType.LESS_THAN;
		}
		else if(guard.contains(">")){
			splitSymbol = ">";
			compType = ComparisonType.GREATER_THAN;
		}
		else {
			pg = new ProcessedGuard(false, "Missing comparison operator: =,<,>,>=,<=");
			return pg; //Error
		}
		String [] arithSplit = guardNoSpaces.split(splitSymbol);
		if (arithSplit.length == 2){
			try{
				int num = Integer.parseInt(arithSplit[1]);
				if (arithSplit[0].contains(nrId)){
					String [] hashSplit = arithSplit[0].split(nrId);
					if(hashSplit.length == 2){
						pg = new ProcessedGuard(true, "Everything ok", hashSplit[1], compType, num);
					}
					else{
						pg = new ProcessedGuard(false,"Guard condition contains # and comparison operator but has not been properly defined.");
					}
				}
				else{
					pg = new ProcessedGuard(false,"Guard condition is missing #");
				}
			}
			catch(NumberFormatException e){
				pg = new ProcessedGuard(false, "Guard does not contain valid number, i.e. " + arithSplit[1] + " is not a valid number");
			}
		}
		else {
			pg = new ProcessedGuard(false, "Guard contains more than one comparison operator");
		}
		return pg;
	}

}
