package org.processmininguc.utils;

import java.util.Collection;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.processmining.models.graphbased.directed.petrinet.InhibitorNet;
import org.processmining.models.graphbased.directed.petrinet.PetrinetEdge;
import org.processmining.models.graphbased.directed.petrinet.PetrinetNode;
import org.processmining.models.graphbased.directed.petrinet.elements.Arc;
import org.processmining.models.graphbased.directed.petrinet.elements.InhibitorArc;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;

import edu.upc.essi.petrinets.uuid.Petrinet;
import edu.upc.essi.petrinets.uuid.PlaceId;
import edu.upc.essi.petrinets.uuid.TransitionId;

public class PetrinetPrinter {

	
	public static void printAll(Map<String,Petrinet> pnMap){
		for(Petrinet pn: pnMap.values()){
			print(pn);
		}
		
	}
	
	public static void printAll(List<Petrinet> pnList){
		for(Petrinet pn: pnList){
			print(pn);
		}
	}
	
	public static void print(InhibitorNet pn){
		String label = pn.getLabel();
		System.out.println("Petrinet Name: " + label);
		System.out.println("****************************");
		Collection<Place> plCollection = pn.getPlaces();
		List<PetrinetNode> visitedNodes = new LinkedList<>();
		Collection<Transition> trCollection = pn.getTransitions();
		//Debugging purposes
		//System.out.println(" ============================ Printing node list before printing details of Petri net  ============================");
		//for(PetrinetNode pnNode: pn.getNodes()){
			//System.out.println("Node ID: "+ pnNode.getId() + "Node label: " + pnNode.getLabel());
		//}
		//System.out.println("End printing");
		for(Place pl: plCollection){
			printPlaceAndTransitions(pn, pl, visitedNodes);
		}
		System.out.println("****************************");
		

	}
	
	public static void print(Petrinet pn){
		String label = pn.getPetrinetLabel();
		System.out.println("Petrinet Name: " + label);
		System.out.println("****************************");
		Collection<PlaceId> plCollection = pn.getPlaces();
		List<PlaceId> visitedPlaces = new LinkedList<>();
		Collection<TransitionId> trCollection = pn.getTransitions();
		//Debugging purposes
		/*System.out.println(" ============================ Printing node list before printing details of Petri net  ============================");
		for(PlaceId place: pn.getPlaces()){
			System.out.println("Place ID: "+ place.getUUIDToString() + "Place label: " + pn.getPlaceLabel(place));
		}
		for(TransitionId trans: pn.getTransitions()){
			System.out.println("Trans ID: "+ trans.getUUIDToString() + "Trans label: " + pn.getTransitionLabel(trans));
		}
		System.out.println("End printing");*/
		for(PlaceId pl: plCollection){
			printPlacesAndTransitions(pn, pl, visitedPlaces);
		}
		System.out.println("****************************");
		

	}
	
	public static void printPlacesAndTransitions(Petrinet pn, PlaceId pl, List<PlaceId> printedPlaces){
		if(!printedPlaces.contains(pl)){
			String representation = pl.getUUIDToString() + " " + pn.getPlaceLabel(pl).concat(" [");
			printedPlaces.add(pl);
			Collection<TransitionId> transCol = pn.getNextTransitions(pl);
			for(TransitionId trans: transCol){
				representation = representation.concat(pn.getTransitionLabel(trans) + "(" + trans.getUUIDToString() + ") ,");
				Collection<PlaceId> placeCol = pn.getNextPlaces(trans);
				representation = representation.concat(" [");
				for(PlaceId place: placeCol){
					representation = representation.concat(pn.getPlaceLabel(place) + ",");
					printPlacesAndTransitions(pn, place, printedPlaces);
				}
				representation = representation.concat("]");
			}
			representation = representation.concat("]");
			System.out.println("Final rep.: " + representation);
		}
	}
	
	public static void printPlaceAndTransitions(InhibitorNet pn, PetrinetNode pl, List<PetrinetNode> printedNodes){
		if(!printedNodes.contains(pl)){
			String representation = pl.getId() + " " + pl.getLabel().concat(" [");
			printedNodes.add(pl);
			Collection<PetrinetEdge<?, ?>> edgeCol = pn.getOutEdges(pl);
			for(PetrinetEdge<?,?> pnEdge: edgeCol){
				PetrinetNode pnNode;
				if(pnEdge instanceof InhibitorArc){
					InhibitorArc arc = (InhibitorArc) pnEdge;
					pnNode = arc.getTarget();
				}
				else {
					Arc arc = (Arc) pnEdge;
					pnNode = arc.getTarget();
					//System.out.println("Target node (should be a transition) label: " + pnNode.getLabel());
				}
				representation = representation.concat(pnNode.getLabel() + "(" + pnNode.getId() + ") ,");
				printedNodes.add(pnNode);
				representation = representation.concat(" [");
				for(PetrinetEdge<?,?> pnEdge2: pn.getOutEdges(pnNode)){
					Arc arc2 = (Arc) pnEdge2;
					PetrinetNode pnNode2 = arc2.getTarget();
					//System.out.println("Target node (should be a place) label: " + pnNode2.getLabel());
					representation = representation.concat(pnNode2.getLabel() + ",");
					printPlaceAndTransitions(pn, pnNode2, printedNodes);
				}
				representation = representation.concat("]");
			}
			representation = representation.concat("]");
			System.out.println("Final rep.: " + representation);
		}
		
	}
	
	public static void printInitialFinalPlaces(Petrinet pn){
		System.out.println("Initial Places");
		for(PlaceId pl: pn.getInitialPlaces()){
			System.out.println("Place " + pn.getPlaceLabel(pl));
		}
		System.out.println("Final Places");
		for(PlaceId pl: pn.getFinalPlaces()){
			System.out.println("Place " + pn.getPlaceLabel(pl));
		}
	}
	
	public static void printNextTransitions(PlaceId pl, Petrinet pn){
		Collection<TransitionId> transCol = pn.getNextTransitions(pl);
		System.out.println("Next transitions of place: " + pn.getPlaceLabel(pl));
		for(TransitionId trans: transCol){
			System.out.println(pn.getTransitionLabel(trans));
		}
		System.out.println("End next transitions");
	}
	

}
