package tests;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;

import org.junit.Test;

import edu.upc.essi.petrinets.uuid.ArcId;
import edu.upc.essi.petrinets.uuid.Petrinet;
import edu.upc.essi.petrinets.uuid.PlaceId;
import edu.upc.essi.petrinets.uuid.TransitionId;

public class PetrinetTest {

	@Test
	public void testPetrinet() {
		Petrinet pnTest = new Petrinet("PetrinetTest");
		assertEquals("Petri name equals 'PetrinetTest'","PetrinetTest", pnTest.getPetrinetLabel());
		assertTrue("No transitions", pnTest.getTransitions().isEmpty());
		assertTrue("No places", pnTest.getPlaces().isEmpty());
		//assertTrue("No arcs from places to transitions",pnTest.get)
	}

	@Test
	public void testAddPlaceStringInt() {
		Petrinet pnTest = new Petrinet("PetrinetTest");
		PlaceId place = pnTest.addPlace("Start",1);
		assertEquals("Place name equals 'Start'","Start",pnTest.getPlaceLabel(place));
		assertEquals("Place token nr equals 1", 1, pnTest.getPlaceTokens(place));
	}

	@Test
	public void testAddPlaceString() {
		Petrinet pnTest = new Petrinet("PetrinetTest");
		PlaceId place = pnTest.addPlace("Start");
		assertEquals("Place name equals 'Start'","Start",pnTest.getPlaceLabel(place));
		assertEquals("Place token nr equals 0", 0, pnTest.getPlaceTokens(place));
	}

	@Test
	public void testAddTransition() {
		Petrinet pnTest = new Petrinet("PetrinetTest");
		//PlaceId place = pnTest.addPlace("Start");
		TransitionId trans = pnTest.addTransition("T1");
		//PlaceId place2 =  pnTest.addPlace("Final");
		assertEquals("Transition name equals 'T1'","T1",pnTest.getTransitionLabel(trans));
	}
	
	@Test
	public void testAddArc(){
		Petrinet pnTest = new Petrinet("PetrinetTest");
		PlaceId place = pnTest.addPlace("Start");
		TransitionId trans = pnTest.addTransition("T1");
		PlaceId place2 =  pnTest.addPlace("Final");
		pnTest.addArc(place, trans);
		pnTest.addArc(trans, place2);
		assertEquals("Outgoing trans from start node should be 1",1,pnTest.getNextTransitions(place).size());
		assertEquals("Outgoing trans from transition 'T1' should be 1",1,pnTest.getNextPlaces(trans).size());
	}

	@Test
	public void testRemoveTransition() {
		Petrinet pnTest = new Petrinet("PetrinetTest");
		PlaceId place = pnTest.addPlace("Start");
		TransitionId trans = pnTest.addTransition("T1");
		PlaceId place2 =  pnTest.addPlace("Final");
		TransitionId trans2 = pnTest.addTransition("T2");
		PlaceId place3 = pnTest.addPlace("MidPlace");
		pnTest.addArc(place, trans);
		pnTest.addArc(trans, place3);
		pnTest.addArc(place3, trans2);
		pnTest.addArc(trans2, place2);
		pnTest.removeTransition(trans2);
		assertEquals("Number of transitions", 1 ,pnTest.getTransitions().size());
		assertEquals("Initial Places", 2 ,pnTest.getInitialPlaces().size());
	}

	@Test
	public void testRemovePlace() {
		Petrinet pnTest = new Petrinet("PetrinetTest");
		PlaceId place = pnTest.addPlace("Start");
		TransitionId trans = pnTest.addTransition("T1");
		PlaceId place2 =  pnTest.addPlace("Final");
		TransitionId trans2 = pnTest.addTransition("T2");
		PlaceId place3 = pnTest.addPlace("MidPlace");
		pnTest.addArc(place, trans);
		pnTest.addArc(trans, place3);
		pnTest.addArc(place3, trans2);
		pnTest.addArc(trans2, place2);
		pnTest.removePlace(place3);
		assertEquals("Number of places", 2 ,pnTest.getPlaces().size());
		assertEquals("Initial Places", 1 ,pnTest.getInitialPlaces().size());
	}
	
	@Test
	public void testGetArcs(){
		Petrinet pnTest = new Petrinet("PetrinetTest");
		PlaceId place = pnTest.addPlace("Start");
		TransitionId trans = pnTest.addTransition("T1");
		PlaceId place2 =  pnTest.addPlace("Final");
		TransitionId trans2 = pnTest.addTransition("T2");
		PlaceId place3 = pnTest.addPlace("MidPlace");
		ArcId arc1 = pnTest.addArc(place, trans);
		ArcId arc2 = pnTest.addArc(trans, place3);
		pnTest.addArc(place3, trans2);
		pnTest.addArc(trans2, place2);
		assertEquals("Number of arcs",4,pnTest.getArcs().size());
		assertTrue("Arc 1 contained in arc list",pnTest.getArcs().contains(arc1));
		assertTrue("Arc 2 contained in arc list", pnTest.getArcs().contains(arc2));
	}

	@Test
	public void testGetPetrinetLabel() {
		//implemented above in Petrinet constructor test
	}

	@Test
	public void testGetPlaces() {
		Petrinet pnTest = new Petrinet("PetrinetTest");
		PlaceId place = pnTest.addPlace("Start");
		TransitionId trans = pnTest.addTransition("T1");
		PlaceId place2 =  pnTest.addPlace("Final");
		pnTest.addArc(place, trans);
		pnTest.addArc(trans, place2);
		assertEquals("Number of places",2,pnTest.getPlaces().size());
	}

	@Test
	public void testGetTransitions() {
		Petrinet pnTest = new Petrinet("PetrinetTest");
		PlaceId place = pnTest.addPlace("Start");
		TransitionId trans = pnTest.addTransition("T1");
		PlaceId place2 =  pnTest.addPlace("Final");
		TransitionId trans2 = pnTest.addTransition("T2");
		PlaceId place3 = pnTest.addPlace("MidPlace");
		pnTest.addArc(place, trans);
		pnTest.addArc(trans, place3);
		pnTest.addArc(place3, trans2);
		pnTest.addArc(trans2, place2);
		assertEquals("Number of transitions",2,pnTest.getTransitions().size());
	}

	@Test
	public void testGetPlaceLabel() {
		Petrinet pnTest = new Petrinet("PetrinetTest");
		PlaceId place = pnTest.addPlace("Start");
		//TransitionId trans = pnTest.addTransition("T1");
		PlaceId place2 =  pnTest.addPlace("Final");
		//TransitionId trans2 = pnTest.addTransition("T2");
		PlaceId place3 = pnTest.addPlace("MidPlace");
		assertEquals("Place name", "Start" ,pnTest.getPlaceLabel(place));
	}

	@Test
	public void testGetPlaceTokens() {
		Petrinet pnTest = new Petrinet("PetrinetTest");
		PlaceId place = pnTest.addPlace("Start");
		//TransitionId trans = pnTest.addTransition("T1");
		PlaceId place2 =  pnTest.addPlace("Final");
		//TransitionId trans2 = pnTest.addTransition("T2");
		PlaceId place3 = pnTest.addPlace("MidPlace");
		assertEquals("Place name", 0 ,pnTest.getPlaceTokens(place));
	}

	@Test
	public void testSetPlaceTokens() {
		Petrinet pnTest = new Petrinet("PetrinetTest");
		PlaceId place = pnTest.addPlace("Start");
		//TransitionId trans = pnTest.addTransition("T1");
		PlaceId place2 =  pnTest.addPlace("Final");
		//TransitionId trans2 = pnTest.addTransition("T2");
		PlaceId place3 = pnTest.addPlace("MidPlace",1);
		assertEquals("Token Nr of place: ", 0 ,pnTest.getPlaceTokens(place));
		assertEquals("Token Nr of place3: ", 1 ,pnTest.getPlaceTokens(place3));
	}

	@Test
	public void testGetTransitionLabel() {
		Petrinet pnTest = new Petrinet("PetrinetTest");
		PlaceId place = pnTest.addPlace("Start");
		TransitionId trans = pnTest.addTransition("T1");
		PlaceId place2 =  pnTest.addPlace("Final");
		TransitionId trans2 = pnTest.addTransition("T2");
		PlaceId place3 = pnTest.addPlace("MidPlace");
		assertEquals("Trans", "T2" ,pnTest.getTransitionLabel(trans2));
		assertEquals("Trans", "T1" ,pnTest.getTransitionLabel(trans));
	}

	@Test
	public void testGetInitialPlaces() {
		Petrinet pnTest = new Petrinet("PetrinetTest");
		PlaceId place = pnTest.addPlace("Start");
		TransitionId trans = pnTest.addTransition("T1");
		PlaceId place2 =  pnTest.addPlace("Final");
		TransitionId trans2 = pnTest.addTransition("T2");
		PlaceId place3 = pnTest.addPlace("MidPlace");
		assertEquals("Initial Places", 3 ,pnTest.getInitialPlaces().size());
		pnTest.addArc(place, trans);
		pnTest.addArc(trans, place3);
		pnTest.addArc(place3, trans2);
		pnTest.addArc(trans2, place2);
		assertEquals("Initial Places", 1 ,pnTest.getInitialPlaces().size());
		pnTest.addArc(trans, place);
		assertEquals("Initial Places", 0 ,pnTest.getInitialPlaces().size());
	}

	@Test
	public void testGetFinalPlaces() {
		Petrinet pnTest = new Petrinet("PetrinetTest");
		PlaceId place = pnTest.addPlace("Start");
		TransitionId trans = pnTest.addTransition("T1");
		PlaceId place2 =  pnTest.addPlace("Final");
		TransitionId trans2 = pnTest.addTransition("T2");
		PlaceId place3 = pnTest.addPlace("MidPlace");
		assertEquals("Final Places", 3 ,pnTest.getFinalPlaces().size());
		pnTest.addArc(place, trans);
		pnTest.addArc(trans, place3);
		pnTest.addArc(place3, trans2);
		pnTest.addArc(trans2, place2);
		assertEquals("Final Places", 1 ,pnTest.getFinalPlaces().size());
		pnTest.addArc(place2, trans);
		assertEquals("Final Places", 0 ,pnTest.getFinalPlaces().size());
	}

	@Test
	public void testGetNextTransitions() {
		Petrinet pnTest = new Petrinet("PetrinetTest");
		PlaceId place = pnTest.addPlace("Start");
		TransitionId trans = pnTest.addTransition("T1");
		PlaceId place2 =  pnTest.addPlace("Final");
		TransitionId trans2 = pnTest.addTransition("T2");
		PlaceId place3 = pnTest.addPlace("MidPlace");
		pnTest.addArc(place, trans);
		pnTest.addArc(trans, place3);
		pnTest.addArc(place3, trans2);
		pnTest.addArc(trans2, place2);
		assertEquals("Number of next transitions Start ",1,pnTest.getNextTransitions(place).size());
		assertEquals("Number of next transitions of MidPlace",1,pnTest.getNextTransitions(place3).size());
		assertEquals("Number of next transitions of Final",0,pnTest.getNextTransitions(place2).size());
	}

	@Test
	public void testGetPreviousTransitions() {
		Petrinet pnTest = new Petrinet("PetrinetTest");
		PlaceId place = pnTest.addPlace("Start");
		TransitionId trans = pnTest.addTransition("T1");
		PlaceId place2 =  pnTest.addPlace("Final");
		TransitionId trans2 = pnTest.addTransition("T2");
		PlaceId place3 = pnTest.addPlace("MidPlace");
		pnTest.addArc(place, trans);
		pnTest.addArc(trans, place3);
		pnTest.addArc(place3, trans2);
		pnTest.addArc(trans2, place2);
		assertEquals("Number of prev transitions Start ",0,pnTest.getPreviousTransitions(place).size());
		assertEquals("Number of prev transitions of MidPlace",1,pnTest.getPreviousTransitions(place3).size());
		assertEquals("Number of prev transitions of Final",1,pnTest.getPreviousTransitions(place2).size());
	}

	@Test
	public void testGetNextPlaces() {
		Petrinet pnTest = new Petrinet("PetrinetTest");
		PlaceId place = pnTest.addPlace("Start");
		TransitionId trans = pnTest.addTransition("T1");
		PlaceId place2 =  pnTest.addPlace("Final");
		TransitionId trans2 = pnTest.addTransition("T2");
		PlaceId place3 = pnTest.addPlace("MidPlace");
		pnTest.addArc(place, trans);
		pnTest.addArc(trans, place3);
		pnTest.addArc(place3, trans2);
		pnTest.addArc(trans2, place2);
		assertEquals("Number of next places of T1 ",1,pnTest.getNextPlaces(trans).size());
		assertEquals("Number of next places of T2",1,pnTest.getNextPlaces(trans2).size());
	}

	@Test
	public void testGetPreviousPlaces() {
		Petrinet pnTest = new Petrinet("PetrinetTest");
		PlaceId place = pnTest.addPlace("Start");
		TransitionId trans = pnTest.addTransition("T1");
		PlaceId place2 =  pnTest.addPlace("Final");
		TransitionId trans2 = pnTest.addTransition("T2");
		PlaceId place3 = pnTest.addPlace("MidPlace");
		pnTest.addArc(place, trans);
		pnTest.addArc(trans, place3);
		pnTest.addArc(place3, trans2);
		pnTest.addArc(trans2, place2);
		assertEquals("Number of previous places of T1 ",1,pnTest.getPreviousPlaces(trans).size());
		assertEquals("Number of previous places of T2 ",1,pnTest.getPreviousPlaces(trans2).size());
	}
	
	@Test
	public void testClonePetrineet(){
		Petrinet pnTest = new Petrinet("PetrinetTest");
		PlaceId place = pnTest.addPlace("Start");
		TransitionId trans = pnTest.addTransition("T1");
		PlaceId place2 =  pnTest.addPlace("Final");
		TransitionId trans2 = pnTest.addTransition("T2");
		PlaceId place3 = pnTest.addPlace("MidPlace");
		pnTest.addArc(place, trans);
		pnTest.addArc(trans, place3);
		pnTest.addArc(place3, trans2);
		pnTest.addArc(trans2, place2);
		Petrinet copiedPN = pnTest.clonePetrinet();
		assertTrue("Checking that the copied PN is another object different from the original", !copiedPN.equals(pnTest));
		assertEquals("Same number of arcs", pnTest.getArcs().size(),copiedPN.getArcs().size());
		assertEquals("Same number of nodes", pnTest.getPlaces().size(), copiedPN.getPlaces().size());
	}

}
